unit SoundMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons, ToolWin, ImgList, mmsystem, ExtCtrls,
  registry, Gauges, PlayerTr;

type
  TSounderForm = class(TForm)
    VolumeLBar: TTrackBar;
    VolumeRBar: TTrackBar;
    Label1: TLabel;
    ToolBar1: TToolBar;
    OpenButton: TToolButton;
    ImageList1: TImageList;
    PlayButton: TToolButton;
    StopButton: TToolButton;
    ToolButton4: TToolButton;
    PouseButton: TToolButton;
    OpenDialog1: TOpenDialog;
    SoundTimer: TTimer;
    Label4: TLabel;
    Label5: TLabel;
    Bevel1: TBevel;
    ToolButton2: TToolButton;
    ExitButton: TToolButton;
    ProgressBar1: TProgressBar;
    Label6: TLabel;
    Label7: TLabel;
    FreqBox: TEdit;
    BitsBox: TEdit;
    ChanelsBox: TEdit;
    PositionBar: TProgressBar;
    ToolButton1: TToolButton;
    ToolButton3: TToolButton;
    procedure FormShow(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure PouseButtonClick(Sender: TObject);
    procedure SoundTimerTimer(Sender: TObject);
    procedure VolumeLBarChange(Sender: TObject);
    procedure PositionBarMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ExitButtonClick(Sender: TObject);
    procedure PositionBarMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ToolButton3Click(Sender: TObject);
  private
    { Private declarations }
    RestartPlay:Boolean;
    procedure FillDevice;
    procedure OpenWaveFile;
    procedure EnableBut(Stat:Boolean);
    procedure InitSoundControls;
    procedure CalculateRestart;
  public
    { Public declarations }
    CyDSounder1:TSoundPlayer;
    WavFileName:String;
    wFileSize,DataSize,DataOffset:Int64;
    w_hData:THandle;
    wfx:TWAVEFORMATEX;
    w_Pause, w_Restart, w_Stop:Boolean;
    w_PlayPosition,w_RestartPosition:Int64;
    w_DeviceID:Integer;
    w_hPlay:HWAVEOUT;
  end;

var
  SounderForm: TSounderForm;

implementation

uses SoundOptionDlg;

{$R *.DFM}

procedure WaveInCheck(res:MMResult);
var
 ErrText:array [0..MAXERRORLENGTH+1] of char;
begin
 if res=MMSYSERR_NOERROR then exit;
 waveInGetErrorText(res,ErrText,Sizeof(ErrText));
 MessageBox(SounderForm.Handle,ErrText, 'Error', MB_OK+MB_ICONEXCLAMATION);
end;

procedure TSounderForm.FillDevice;
var
 Num,i:Integer;
 WaveCaps:TWaveOutCaps;
 pwoc:TWaveOutCaps;
begin
 SoundOptionForm.AudioDeviceCBox.Items.Clear;
 Num:=waveOutGetNumDevs;
 for i:=0 to Num-1 do
  begin
   WaveOutGetDevCaps(i,@WaveCaps,sizeof(WaveCaps));
   SoundOptionForm.AudioDeviceCBox.Items.Add(WaveCaps.szPName);
  end;
 SoundOptionForm.AudioDeviceCBox.Items.Add('Automatic select audio device');

 SoundOptionForm.RecDeviceCBox.Items.Clear;
 Num:=waveInGetNumDevs;
 for i:=0 to Num-1 do
  begin
   WaveInGetDevCaps(i,@WaveCaps,sizeof(WaveCaps));
   SoundOptionForm.RecDeviceCBox.Items.Add(WaveCaps.szPName);
  end;
 SoundOptionForm.RecDeviceCBox.Items.Add('Automatic select record device');

 waveOutGetDevCaps(w_DeviceID, @pwoc, sizeof(pwoc));
 if (pwoc.dwSupport and WAVECAPS_VOLUME)>0 then
  begin
   VolumeLBar.Enabled:=true;
   VolumeRBar.Enabled:=true;
  end;
end;

procedure TSounderForm.FormShow(Sender: TObject);
begin
 EnableBut(false);
 FillDevice;
 RestartPlay:=false;
 SoundOptionForm.AudioDeviceCBox.ItemIndex:=SoundOptionForm.AudioDeviceCBox.Items.Count-1;
 SoundOptionForm.RecDeviceCBox.ItemIndex:=SoundOptionForm.RecDeviceCBox.Items.Count-1;
end;

procedure TSounderForm.OpenButtonClick(Sender: TObject);
begin
 StopButtonClick(nil);
 DataSize:=0;
 if OpenDialog1.Execute then
  OpenWaveFile;

 WavFileName:=OpenDialog1.FileName;
 Caption:='Sounder - '+ExtractFileName(WavFileName);

 if wfx.nChannels=1 then
  ChanelsBox.Text:=''
 else
  ChanelsBox.Text:='';
 FreqBox.Text:=IntToStr(wfx.nSamplesPerSec);
 BitsBox.Text:=IntToStr(wfx.wBitsPerSample);
 Label7.Caption:=IntToStr(wfx.nAvgBytesPerSec);
end;

procedure TSounderForm.OpenWaveFile;
var
 hFile:THandle;
 Str:array [0..255] of char;
 TempStr:String;
 BytesReaded,Size:DWORD;
begin
 if w_hData<>0 then CloseHandle(w_hData);
 hFile:=CreateFile(PChar(OpenDialog1.FileName),GENERIC_READ,FILE_SHARE_READ,nil,OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE,0);
 if hFile=INVALID_HANDLE_VALUE then exit;
 try
  wFileSize:=GetFileSize(hFile,nil);
  try
   SetFilePointer(hFile,8,nil,FILE_BEGIN);
   ReadFile(hFile,Str,8,BytesReaded,nil);
   Str[7]:=#0;
   TempStr:=Str;
   if TempStr<>'WAVEfmt' then
    begin
     Application.MessageBox(' ','Error',MB_OK+MB_ICONINFORMATION);
     CloseHandle(hFile);
     exit;
    end;
   ReadFile(hFile,Size,Sizeof(Size),BytesReaded,nil);
   ZeroMemory(@wfx,sizeof(wfx));

   if Size=sizeof(PCMWAVEFORMAT) then
    begin
     ReadFile(hFile,wfx,Size,BytesReaded,nil);
    end
   else
    begin
     MessageBox(Handle, '  ', 'Error', MB_OK+MB_ICONEXCLAMATION);
     exit;
    end;

   wfx.cbSize:=0;
   ReadFile(hFile,Str,4,BytesReaded,nil);
   ReadFile(hFile,DataSize,4,BytesReaded,nil);

   DataOffset:=SetFilePointer(hFile,0,nil,FILE_CURRENT);

   w_hData:=CreateFilemapping(hFile,nil,PAGE_READONLY,0,Integer(wFileSize),nil);
   CloseHandle(hFile);
   hFile:=INVALID_HANDLE_VALUE;
  except
   Application.MessageBox('  ','Error',MB_OK+MB_ICONINFORMATION);
   CloseHandle(hFile);
   hFile:=INVALID_HANDLE_VALUE;
  end;
 finally
  if hFile<>INVALID_HANDLE_VALUE then CloseHandle(hFile);

  if DataSize>0 then
   begin
    EnableBut(true);
    InitSoundControls;
   end
  else
   EnableBut(false);
 end;
end;

procedure TSounderForm.PlayButtonClick(Sender: TObject);
begin
 if ((w_Pause)and (w_hPlay<>0)) then
  begin
   waveOutRestart(w_hPlay);
   w_Pause:=not w_Pause;
   exit;
  end;

 if w_hPlay<>0 then StopButtonClick(nil);
 SoundTimer.Enabled:=true;
 w_Pause:=false; w_Restart:=false; w_Stop:=false;
 w_PlayPosition:=PositionBar.Position*1000;
 w_RestartPosition:=PositionBar.Position*1000;

 w_DeviceID:=SoundOptionForm.AudioDeviceCBox.ItemIndex;
 if SoundOptionForm.AudioDeviceCBox.ItemIndex=SoundOptionForm.AudioDeviceCBox.Items.Count-1 then
  w_DeviceID:=Integer(WAVE_MAPPER);

 CyDSounder1:=TSoundPlayer.Create(true);
 CyDSounder1.Priority:=tpHigher;
 CyDSounder1.Resume;
end;

procedure TSounderForm.StopButtonClick(Sender: TObject);
begin
 if w_hPlay<>0 then
  begin
   w_Stop:=true;
   waveOutReset(w_hPlay);
   SoundTimer.Enabled:=false;
  end;
end;

procedure TSounderForm.PouseButtonClick(Sender: TObject);
begin
 if w_hPlay=0 then exit;
 waveOutPause(w_hPlay);
 w_Pause:=not w_Pause;
end;

procedure TSounderForm.SoundTimerTimer(Sender: TObject);
var
 mmt:MMTIME;
begin
 mmt.wType:=TIME_BYTES;
 waveOutGetPosition(w_hPlay, @mmt,sizeof(mmt));
 PositionBar.Position:=round((mmt.cb+w_RestartPosition)/1000);
end;

procedure TSounderForm.EnableBut(Stat:Boolean);
begin
 PlayButton.Enabled:=Stat;
 StopButton.Enabled:=Stat;
 PouseButton.Enabled:=Stat;
 PositionBar.Max:=round(DataSize/1000);
end;

procedure TSounderForm.InitSoundControls;
var
 Value:DWord;
begin
 StopButtonClick(nil);
 waveOutOpen(@w_hPlay, WAVE_MAPPER, @wfx, 0, 0,CALLBACK_NULL);
 waveOutGetPlayBackRate(w_hPlay, @Value);
 waveOutGetPitch(w_hPlay, @Value);
 waveOutClose(w_hPlay);
 w_hPlay:=0;
end;

procedure TSounderForm.VolumeLBarChange(Sender: TObject);
var
 LValue,RValue:DWord;
begin
 LValue:=$FFFF-VolumeLBar.Position;
 RValue:=$FFFF-VolumeRBar.Position;
 waveOutSetVolume(0, (RValue SHL 16)+LValue);
end;

procedure TSounderForm.PositionBarMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if w_hPlay=0 then exit;
 RestartPlay:=true;
 PouseButtonClick(nil);
end;

procedure TSounderForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 StopButtonClick(nil);
end;

procedure TSounderForm.ExitButtonClick(Sender: TObject);
begin
 Close;
end;

procedure TSounderForm.PositionBarMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 CalculateRestart;
end;

procedure TSounderForm.CalculateRestart;
begin
 if PositionBar.Position<DataSize then
  begin
   w_Restart:=true;
   w_RestartPosition:=PositionBar.Position;
   w_RestartPosition:=w_RestartPosition-(w_RestartPosition mod wfx.nBlockAlign);
  end
 else
  StopButtonClick(nil);
 if RestartPlay=true then
  PouseButtonClick(nil);
 RestartPlay:=false;  
end;

procedure TSounderForm.ToolButton3Click(Sender: TObject);
begin
 SoundOptionForm.ShowModal;
end;

end.
